//LCD.c

#include "LCD.h"
#include "main.h"

#define FCY 64000000LL            //rajel definilsa
#include <libpic30.h>


void LCDWrite(unsigned char Data, unsigned char Register);
void WriteString(char* str);

void LCDInit()
{
	__delay_ms(100);
	LCDWrite(0b00110000, 0);		//set 8 bit operation and basic instruction set 
	LCDWrite(LCD_DISPLAY_ON, 0);	//display on
	LCDWrite(LCD_CLEAR, 0);			//display clear
	__delay_ms(2);
	LCDWrite(LCD_ENTRY_MODE_SET, 0);		//entry mode set 
	LCDWrite(LCD_EXTENDED_INSTRUCTION_SET, 0);		//extended instruction set 
	LCDWrite(LCD_EXTENDED_INSTRUCTION_SET, 0);		//extended instruction set again
	LCDWrite(LCD_GRAPHICS_MODE_ON , 0);		//set graphics mode on
	LCDClearGraphics();
	LCDClearText();	
}


void LCDDrawGraphics(char* Image)
{

	unsigned char x, y;
	for(y = 0; y < 64; y++)
	{
		if(y < 32)
		{
			for(x = 0; x < 8; x++)						// Draws top half of the screen.
			{											// In extended instruction mode, vertical and horizontal coordinates must be specified before sending data in.
				LCDWrite(0x80 | y, 0);					// Vertical coordinate of the screen is specified first. (0-31)
    			LCDWrite(0x80 | x, 0);					// Then horizontal coordinate of the screen is specified. (0-8)
				LCDWrite(Image[2*x + 16*y], 1);			// Data to the upper byte is sent to the coordinate.
				LCDWrite(Image[2*x+1 + 16*y], 1);		// Data to the lower byte is sent to the coordinate.
			}
		}
		else
		{
			for(x = 0; x < 8; x++)					// Draws bottom half of the screen.
			{										// Actions performed as same as the upper half screen.
				LCDWrite(0x80 | (y-32), 0);			// Vertical coordinate must be scaled back to 0-31 as it is dealing with another half of the screen.
    			LCDWrite(0x88 | x, 0);
				LCDWrite(Image[2*x + 16*y], 1);
				LCDWrite(Image[2*x+1 + 16*y], 1);
			}
		}
		
	}
}

void LCDClearGraphics()
{
	unsigned char x, y;
	for(y = 0; y < 64; y++)
	{
		if(y < 32)
		{
			LCDWrite(0x80 | y, 0);
    		LCDWrite(0x80, 0);
		}
		else
		{
			LCDWrite(0x80 | (y-32), 0);
    		LCDWrite(0x88, 0);
		}
		for(x = 0; x < 8; x++)
		{
			LCDWrite(0x00, 1);
			LCDWrite(0x00, 1);
		}
	}
}

void LCDWriteAngle(char* str,char* str2)
{
	LCDWrite(LCD_GRAPHICS_MODE_OFF,0);
	 __delay_us(72);
	LCDWrite(LCD_SELECT_LINE_1 | 0x00,0);
	 __delay_us(72);
	WriteString(str);
	LCDWrite(LCD_SELECT_LINE_2 | 0x00,0);
	 __delay_us(72);
	WriteString(str2);
	LCDWrite(LCD_EXTENDED_INSTRUCTION_SET, 0);			 
	LCDWrite(LCD_EXTENDED_INSTRUCTION_SET, 0);			
	LCDWrite(LCD_GRAPHICS_MODE_ON , 0);	
	 __delay_us(100);	
}


void WriteString(char* str)
{
	while(*str != 0)
	{
		LCDWrite(*str, 1);
		str++;
	}

}




void LCDWrite(unsigned char Data, unsigned char Register)
{
	char Temp;
	if(Register == 1)
	{
		Temp = 0b11111010;
	}
	else
	{
		Temp = 0b11111000;
	}
	SPISend(&Temp);
	Temp = Data & 0b11110000;
	SPISend(&Temp);
	Temp = Data & 0b00001111;
	Temp = Temp << 4;
	SPISend(&Temp);
	__delay_us(20);
}



void IntToChar(int number, char *chr)	//Convert integer to char
{
	unsigned int Divisor = 10000;
	if(number == 0)
	{
		*chr = '0';
		chr++;
		*chr = 0;
		return;
	}
	if((number & 0b1000000000000000) == 0b1000000000000000)
	{
		number = number * -1;
		*chr = '-';
		chr++;
	}

	*chr = '0';
	while(*chr == '0')
	{
		*chr = number/Divisor + 48;
		number = number%Divisor;
		Divisor = Divisor/10;
	}
	if(Divisor == 0)
	{
		chr++;
		*chr = 0;
	}
	while(*chr != 0)
	{
		chr++;
		*chr = number/Divisor + 48;
		number = number%Divisor;
		if(Divisor == 1)
		{
			chr++;
			*chr = 0;
		}
		Divisor = Divisor/10;
	}
}

void LCDClearText()
{
	LCDWrite(LCD_GRAPHICS_MODE_OFF,0);
	 __delay_us(100);
	LCDWrite(LCD_SELECT_LINE_1,0);
	LCDWrite(LCD_CLEAR,0);
	 __delay_ms(2);
	LCDWrite(LCD_EXTENDED_INSTRUCTION_SET, 0);		 
	LCDWrite(LCD_EXTENDED_INSTRUCTION_SET, 0);		
	LCDWrite(LCD_GRAPHICS_MODE_ON , 0);		
}





